@@ -40,6 +40,7 @@
 //M*/
 
 #include "precomp.hpp"
+#include "opencv2/calib3d/calib3d_c.h"
 
 // cvCorrectMatches function is Copyright (C) 2009, Jostein Austvik Jacobsen.
 // cvTriangulatePoints function is derived from icvReconstructPointsFor3View, originally by Valery Mosyagin.
@@ -62,8 +63,7 @@ cvTriangulatePoints(CvMat* projMatr1, CvMat* projMatr2, CvMat* projPoints1, CvMa
       !CV_IS_MAT(points4D) )
       CV_Error( CV_StsUnsupportedFormat, "Input parameters must be matrices" );
 
-    int numPoints;
-    numPoints = projPoints1->cols;
+    int numPoints = projPoints1->cols;
 
     if( numPoints < 1 )
         CV_Error( CV_StsOutOfRange, "Number of points must be more than zero" );
@@ -81,59 +81,40 @@ cvTriangulatePoints(CvMat* projMatr1, CvMat* projMatr2, CvMat* projPoints1, CvMa
        projMatr2->cols != 4 || projMatr2->rows != 3)
         CV_Error( CV_StsUnmatchedSizes, "Size of projection matrices must be 3x4" );
 
-    CvMat matrA;
-    double matrA_dat[24];
-    matrA = cvMat(6,4,CV_64F,matrA_dat);
+    // preallocate SVD matrices on stack
+    cv::Matx<double, 4, 4> matrA;
+    cv::Matx<double, 4, 4> matrU;
+    cv::Matx<double, 4, 1> matrW;
+    cv::Matx<double, 4, 4> matrV;
 
-    //CvMat matrU;
-    CvMat matrW;
-    CvMat matrV;
-    //double matrU_dat[9*9];
-    double matrW_dat[6*4];
-    double matrV_dat[4*4];
-
-    //matrU = cvMat(6,6,CV_64F,matrU_dat);
-    matrW = cvMat(6,4,CV_64F,matrW_dat);
-    matrV = cvMat(4,4,CV_64F,matrV_dat);
-
-    CvMat* projPoints[2];
-    CvMat* projMatrs[2];
-
-    projPoints[0] = projPoints1;
-    projPoints[1] = projPoints2;
-
-    projMatrs[0] = projMatr1;
-    projMatrs[1] = projMatr2;
+    CvMat* projPoints[2] = {projPoints1, projPoints2};
+    CvMat* projMatrs[2] = {projMatr1, projMatr2};
 
     /* Solve system for each point */
-    int i,j;
-    for( i = 0; i < numPoints; i++ )/* For each point */
+    for( int i = 0; i < numPoints; i++ )/* For each point */
     {
         /* Fill matrix for current point */
-        for( j = 0; j < 2; j++ )/* For each view */
+        for( int j = 0; j < 2; j++ )/* For each view */
         {
             double x,y;
             x = cvmGet(projPoints[j],0,i);
             y = cvmGet(projPoints[j],1,i);
             for( int k = 0; k < 4; k++ )
             {
-                cvmSet(&matrA, j*3+0, k, x * cvmGet(projMatrs[j],2,k) -     cvmGet(projMatrs[j],0,k) );
-                cvmSet(&matrA, j*3+1, k, y * cvmGet(projMatrs[j],2,k) -     cvmGet(projMatrs[j],1,k) );
-                cvmSet(&matrA, j*3+2, k, x * cvmGet(projMatrs[j],1,k) - y * cvmGet(projMatrs[j],0,k) );
+                matrA(j*2+0, k) = x * cvmGet(projMatrs[j],2,k) - cvmGet(projMatrs[j],0,k);
+                matrA(j*2+1, k) = y * cvmGet(projMatrs[j],2,k) - cvmGet(projMatrs[j],1,k);
             }
         }
         /* Solve system for current point */
-        {
-            cvSVD(&matrA,&matrW,0,&matrV,CV_SVD_V_T);
-            
-            /* Copy computed point */
-            cvmSet(points4D,0,i,cvmGet(&matrV,3,0));/* X */
-            cvmSet(points4D,1,i,cvmGet(&matrV,3,1));/* Y */
-            cvmSet(points4D,2,i,cvmGet(&matrV,3,2));/* Z */
-            cvmSet(points4D,3,i,cvmGet(&matrV,3,3));/* W */
-        }
+        cv::SVD::compute(matrA, matrW, matrU, matrV);
+
+        /* Copy computed point */
+        cvmSet(points4D,0,i,matrV(3,0));/* X */
+        cvmSet(points4D,1,i,matrV(3,1));/* Y */
+        cvmSet(points4D,2,i,matrV(3,2));/* Z */
+        cvmSet(points4D,3,i,matrV(3,3));/* W */
     }
-    
+
 #if 0
     double err = 0;
     /* Points was reconstructed. Try to reproject points */
@@ -143,34 +124,34 @@ cvTriangulatePoints(CvMat* projMatr1, CvMat* projMatr2, CvMat* projPoints1, CvMa
         CvMat point3D;
         double point3D_dat[4];
         point3D = cvMat(4,1,CV_64F,point3D_dat);
-        
+
         CvMat point2D;
         double point2D_dat[3];
         point2D = cvMat(3,1,CV_64F,point2D_dat);
-        
+
         for( i = 0; i < numPoints; i++ )
         {
             double W = cvmGet(points4D,3,i);
-            
+
             point3D_dat[0] = cvmGet(points4D,0,i)/W;
             point3D_dat[1] = cvmGet(points4D,1,i)/W;
             point3D_dat[2] = cvmGet(points4D,2,i)/W;
             point3D_dat[3] = 1;
-            
+
             /* !!! Project this point for each camera */
             for( int currCamera = 0; currCamera < 2; currCamera++ )
             {
                 cvMatMul(projMatrs[currCamera], &point3D, &point2D);
-                
+
                 float x,y;
                 float xr,yr,wr;
                 x = (float)cvmGet(projPoints[currCamera],0,i);
                 y = (float)cvmGet(projPoints[currCamera],1,i);
-                
+
                 wr = (float)point2D_dat[2];
                 xr = (float)(point2D_dat[0]/wr);
                 yr = (float)(point2D_dat[1]/wr);
-                
+
                 float deltaX,deltaY;
                 deltaX = (float)fabs(x-xr);
                 deltaY = (float)fabs(y-yr);
@@ -210,7 +191,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
     cv::Ptr<CvMat> result;
     cv::Ptr<CvMat> points1, points2;
     cv::Ptr<CvMat> F;
-	
+
     if (!CV_IS_MAT(F_) || !CV_IS_MAT(points1_) || !CV_IS_MAT(points2_) )
         CV_Error( CV_StsUnsupportedFormat, "Input parameters must be matrices" );
     if (!( F_->cols == 3 && F_->rows == 3))
@@ -237,34 +218,34 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         if (CV_MAT_CN(new_points2->type) != 2)
             CV_Error( CV_StsUnsupportedFormat, "The second output matrix must have two channels; one for x and one for y" );
     }
-	
+
     // Make sure F uses double precision
-    F = cvCreateMat(3,3,CV_64FC1);
+    F.reset(cvCreateMat(3,3,CV_64FC1));
     cvConvert(F_, F);
-	
+
     // Make sure points1 uses double precision
-    points1 = cvCreateMat(points1_->rows,points1_->cols,CV_64FC2);
+    points1.reset(cvCreateMat(points1_->rows,points1_->cols,CV_64FC2));
     cvConvert(points1_, points1);
-	
+
     // Make sure points2 uses double precision
-    points2 = cvCreateMat(points2_->rows,points2_->cols,CV_64FC2);
+    points2.reset(cvCreateMat(points2_->rows,points2_->cols,CV_64FC2));
     cvConvert(points2_, points2);
-	
-    tmp33 = cvCreateMat(3,3,CV_64FC1);
-    tmp31 = cvCreateMat(3,1,CV_64FC1), tmp31_2 = cvCreateMat(3,1,CV_64FC1);
-    T1i = cvCreateMat(3,3,CV_64FC1), T2i = cvCreateMat(3,3,CV_64FC1);
-    R1 = cvCreateMat(3,3,CV_64FC1), R2 = cvCreateMat(3,3,CV_64FC1);
-    TFT = cvCreateMat(3,3,CV_64FC1), TFTt = cvCreateMat(3,3,CV_64FC1), RTFTR = cvCreateMat(3,3,CV_64FC1);
-    U = cvCreateMat(3,3,CV_64FC1);
-    S = cvCreateMat(3,3,CV_64FC1);
-    V = cvCreateMat(3,3,CV_64FC1);
-    e1 = cvCreateMat(3,1,CV_64FC1), e2 = cvCreateMat(3,1,CV_64FC1);
-	
+
+    tmp33.reset(cvCreateMat(3,3,CV_64FC1));
+    tmp31.reset(cvCreateMat(3,1,CV_64FC1)), tmp31_2.reset(cvCreateMat(3,1,CV_64FC1));
+    T1i.reset(cvCreateMat(3,3,CV_64FC1)), T2i.reset(cvCreateMat(3,3,CV_64FC1));
+    R1.reset(cvCreateMat(3,3,CV_64FC1)), R2.reset(cvCreateMat(3,3,CV_64FC1));
+    TFT.reset(cvCreateMat(3,3,CV_64FC1)), TFTt.reset(cvCreateMat(3,3,CV_64FC1)), RTFTR.reset(cvCreateMat(3,3,CV_64FC1));
+    U.reset(cvCreateMat(3,3,CV_64FC1));
+    S.reset(cvCreateMat(3,3,CV_64FC1));
+    V.reset(cvCreateMat(3,3,CV_64FC1));
+    e1.reset(cvCreateMat(3,1,CV_64FC1)), e2.reset(cvCreateMat(3,1,CV_64FC1));
+
     double x1, y1, x2, y2;
     double scale;
     double f1, f2, a, b, c, d;
-    polynomial = cvCreateMat(1,7,CV_64FC1);
-    result = cvCreateMat(1,6,CV_64FC2);
+    polynomial.reset(cvCreateMat(1,7,CV_64FC1));
+    result.reset(cvCreateMat(1,6,CV_64FC2));
     double t_min, s_val, t, s;
     for (int p = 0; p < points1->cols; ++p) {
         // Replace F by T2-t * F * T1-t
@@ -272,7 +253,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         y1 = points1->data.db[p*2+1];
         x2 = points2->data.db[p*2];
         y2 = points2->data.db[p*2+1];
-		
+
         cvSetZero(T1i);
         cvSetReal2D(T1i,0,0,1);
         cvSetReal2D(T1i,1,1,1);
@@ -288,7 +269,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         cvGEMM(T2i,F,1,0,0,tmp33,CV_GEMM_A_T);
         cvSetZero(TFT);
         cvGEMM(tmp33,T1i,1,0,0,TFT);
-        
+
         // Compute the right epipole e1 from F * e1 = 0
         cvSetZero(U);
         cvSetZero(S);
@@ -303,7 +284,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
             cvSetReal2D(e1,1,0,-cvGetReal2D(e1,1,0));
             cvSetReal2D(e1,2,0,-cvGetReal2D(e1,2,0));
         }
-		
+
         // Compute the left epipole e2 from e2' * F = 0  =>  F' * e2 = 0
         cvSetZero(TFTt);
         cvTranspose(TFT, TFTt);
@@ -321,7 +302,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
             cvSetReal2D(e2,1,0,-cvGetReal2D(e2,1,0));
             cvSetReal2D(e2,2,0,-cvGetReal2D(e2,2,0));
         }
-		
+
         // Replace F by R2 * F * R1'
         cvSetZero(R1);
         cvSetReal2D(R1,0,0,cvGetReal2D(e1,0,0));
@@ -337,7 +318,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         cvSetReal2D(R2,2,2,1);
         cvGEMM(R2,TFT,1,0,0,tmp33);
         cvGEMM(tmp33,R1,1,0,0,RTFTR,CV_GEMM_B_T);
-		
+
         // Set f1 = e1(3), f2 = e2(3), a = F22, b = F23, c = F32, d = F33
         f1 = cvGetReal2D(e1,2,0);
         f2 = cvGetReal2D(e2,2,0);
@@ -345,7 +326,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         b = cvGetReal2D(RTFTR,1,2);
         c = cvGetReal2D(RTFTR,2,1);
         d = cvGetReal2D(RTFTR,2,2);
-		
+
         // Form the polynomial g(t) = k6*t⁶ + k5*t⁵ + k4*t⁴ + k3*t³ + k2*t² + k1*t + k0
         // from f1, f2, a, b, c and d
         cvSetReal2D(polynomial,0,6,( +b*c*c*f1*f1*f1*f1*a-a*a*d*f1*f1*f1*f1*c ));
@@ -355,11 +336,11 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         cvSetReal2D(polynomial,0,2,( +4*a*b*b*b+4*b*b*f2*f2*c*d+4*f2*f2*f2*f2*c*d*d*d-a*a*d*c+b*c*c*a+4*a*b*f2*f2*d*d-2*a*d*d*f1*f1*b+2*b*b*c*f1*f1*d ));
         cvSetReal2D(polynomial,0,1,( +f2*f2*f2*f2*d*d*d*d+b*b*b*b+2*b*b*f2*f2*d*d-a*a*d*d+b*b*c*c ));
         cvSetReal2D(polynomial,0,0,( -a*d*d*b+b*b*c*d ));
-		
+
         // Solve g(t) for t to get 6 roots
         cvSetZero(result);
         cvSolvePoly(polynomial, result, 100, 20);
-		
+
         // Evaluate the cost function s(t) at the real part of the 6 roots
         t_min = DBL_MAX;
         s_val = 1./(f1*f1) + (c*c)/(a*a+f2*f2*c*c);
@@ -371,7 +352,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
                 t_min = t;
             }
         }
-		
+
         // find the optimal x1 and y1 as the points on l1 and l2 closest to the origin
         tmp31->data.db[0] = t_min*t_min*f1;
         tmp31->data.db[1] = t_min;
@@ -383,7 +364,7 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         cvGEMM(tmp33,tmp31,1,0,0,tmp31_2);
         x1 = tmp31_2->data.db[0];
         y1 = tmp31_2->data.db[1];
-		
+
         tmp31->data.db[0] = f2*pow(c*t_min+d,2);
         tmp31->data.db[1] = -(a*t_min+b)*(c*t_min+d);
         tmp31->data.db[2] = f2*f2*pow(c*t_min+d,2) + pow(a*t_min+b,2);
@@ -394,14 +375,14 @@ cvCorrectMatches(CvMat *F_, CvMat *points1_, CvMat *points2_, CvMat *new_points1
         cvGEMM(tmp33,tmp31,1,0,0,tmp31_2);
         x2 = tmp31_2->data.db[0];
         y2 = tmp31_2->data.db[1];
-		
+
         // Return the points in the matrix format that the user wants
         points1->data.db[p*2] = x1;
         points1->data.db[p*2+1] = y1;
         points2->data.db[p*2] = x2;
         points2->data.db[p*2+1] = y2;
     }
-    
+
     if( new_points1 )
         cvConvert( points1, new_points1 );
     if( new_points2 )
@@ -412,9 +393,17 @@ void cv::triangulatePoints( InputArray _projMatr1, InputArray _projMatr2,
                             InputArray _projPoints1, InputArray _projPoints2,
                             OutputArray _points4D )
 {
+    CV_INSTRUMENT_REGION()
+
     Mat matr1 = _projMatr1.getMat(), matr2 = _projMatr2.getMat();
     Mat points1 = _projPoints1.getMat(), points2 = _projPoints2.getMat();
 
+    if((points1.rows == 1 || points1.cols == 1) && points1.channels() == 2)
+        points1 = points1.reshape(1, static_cast<int>(points1.total())).t();
+
+    if((points2.rows == 1 || points2.cols == 1) && points2.channels() == 2)
+        points2 = points2.reshape(1, static_cast<int>(points2.total())).t();
+
     CvMat cvMatr1 = matr1, cvMatr2 = matr2;
     CvMat cvPoints1 = points1, cvPoints2 = points2;
 
@@ -427,8 +416,11 @@ void cv::triangulatePoints( InputArray _projMatr1, InputArray _projMatr2,
 void cv::correctMatches( InputArray _F, InputArray _points1, InputArray _points2,
                          OutputArray _newPoints1, OutputArray _newPoints2 )
 {
+    CV_INSTRUMENT_REGION()
+
     Mat F = _F.getMat();
     Mat points1 = _points1.getMat(), points2 = _points2.getMat();
+
     CvMat cvPoints1 = points1, cvPoints2 = points2;
     CvMat cvF = F;
 
