@@ -22,12 +22,16 @@ PERF_TEST_P( Size_Depth_Channels, split,
     int depth = get<1>(GetParam());
     int channels = get<2>(GetParam());
 
-    Mat m(sz, CV_MAKETYPE(depth, channels));    
+    Mat m(sz, CV_MAKETYPE(depth, channels));
     randu(m, 0, 255);
 
     vector<Mat> mv;
+    int runs = (sz.width <= 640) ? 8 : 1;
+    TEST_CYCLE_MULTIRUN(runs) split(m, (vector<Mat>&)mv);
 
-    TEST_CYCLE() split(m, (vector<Mat>&)mv);
-
+#if defined (__aarch64__)
+    SANITY_CHECK(mv, 2e-5);
+#else
     SANITY_CHECK(mv, 1e-12);
+#endif
 }
