@@ -9,7 +9,7 @@ using std::tr1::get;
 enum { TYPE_5_8 =FastFeatureDetector::TYPE_5_8, TYPE_7_12 = FastFeatureDetector::TYPE_7_12, TYPE_9_16 = FastFeatureDetector::TYPE_9_16 };
 CV_ENUM(FastType, TYPE_5_8, TYPE_7_12, TYPE_9_16)
 
-typedef std::tr1::tuple<String, FastType> File_Type_t;
+typedef std::tr1::tuple<string, FastType> File_Type_t;
 typedef perf::TestBaseWithParam<File_Type_t> fast;
 
 #define FAST_IMAGES \
@@ -18,10 +18,10 @@ typedef perf::TestBaseWithParam<File_Type_t> fast;
 
 PERF_TEST_P(fast, detect, testing::Combine(
                             testing::Values(FAST_IMAGES),
-                            testing::ValuesIn(FastType::all())
+                            FastType::all()
                           ))
 {
-    String filename = getDataPath(get<0>(GetParam()));
+    string filename = getDataPath(get<0>(GetParam()));
     int type = get<1>(GetParam());
     Mat frame = imread(filename, IMREAD_GRAYSCALE);
 
@@ -30,15 +30,11 @@ PERF_TEST_P(fast, detect, testing::Combine(
 
     declare.in(frame);
 
-    Ptr<FeatureDetector> fd = Algorithm::create<FeatureDetector>("Feature2D.FASTX");
-    ASSERT_FALSE( fd == 0 );
-    fd->set("threshold", 20);
-    fd->set("nonmaxSuppression", true);
-    fd->set("type", type);
+    Ptr<FeatureDetector> fd = FastFeatureDetector::create(20, true, type);
+    ASSERT_FALSE( fd.empty() );
     vector<KeyPoint> points;
 
     TEST_CYCLE() fd->detect(frame, points);
 
     SANITY_CHECK_KEYPOINTS(points);
 }
-
