@@ -41,7 +41,7 @@
 
 #include "precomp.hpp"
 #include <map>
-#include "opencv2/core/opengl_interop.hpp"
+#include "opencv2/core/opengl.hpp"
 
 // in later times, use this file as a dispatcher to implementations like cvcap.cpp
 
@@ -65,7 +65,10 @@ CV_IMPL void cvSetWindowProperty(const char* name, int prop_id, double prop_valu
             cvSetModeWindow_CARBON(name,prop_value);
         #elif defined (HAVE_COCOA)
             cvSetModeWindow_COCOA(name,prop_value);
+        #elif defined (WINRT)
+            cvSetModeWindow_WinRT(name, prop_value);
         #endif
+
     break;
 
     case CV_WND_PROP_AUTOSIZE:
@@ -104,6 +107,8 @@ CV_IMPL double cvGetWindowProperty(const char* name, int prop_id)
             return cvGetModeWindow_CARBON(name);
         #elif defined (HAVE_COCOA)
             return cvGetModeWindow_COCOA(name);
+        #elif defined (WINRT)
+            return cvGetModeWindow_WinRT(name);
         #else
             return -1;
         #endif
@@ -148,17 +153,25 @@ CV_IMPL double cvGetWindowProperty(const char* name, int prop_id)
         #endif
     break;
 
+    case CV_WND_PROP_VISIBLE:
+        #if defined (HAVE_QT)
+            return cvGetPropVisible_QT(name);
+        #else
+            return -1;
+        #endif
+    break;
+
     default:
         return -1;
     }
 }
 
-void cv::namedWindow( const string& winname, int flags )
+void cv::namedWindow( const String& winname, int flags )
 {
     cvNamedWindow( winname.c_str(), flags );
 }
 
-void cv::destroyWindow( const string& winname )
+void cv::destroyWindow( const String& winname )
 {
     cvDestroyWindow( winname.c_str() );
 }
@@ -168,32 +181,47 @@ void cv::destroyAllWindows()
     cvDestroyAllWindows();
 }
 
-void cv::resizeWindow( const string& winname, int width, int height )
+void cv::resizeWindow( const String& winname, int width, int height )
 {
     cvResizeWindow( winname.c_str(), width, height );
 }
 
-void cv::moveWindow( const string& winname, int x, int y )
+void cv::moveWindow( const String& winname, int x, int y )
 {
     cvMoveWindow( winname.c_str(), x, y );
 }
 
-void cv::setWindowProperty(const string& winname, int prop_id, double prop_value)
+void cv::setWindowProperty(const String& winname, int prop_id, double prop_value)
 {
     cvSetWindowProperty( winname.c_str(), prop_id, prop_value);
 }
 
-double cv::getWindowProperty(const string& winname, int prop_id)
+double cv::getWindowProperty(const String& winname, int prop_id)
 {
     return cvGetWindowProperty(winname.c_str(), prop_id);
 }
 
-int cv::waitKey(int delay)
+int cv::waitKeyEx(int delay)
 {
     return cvWaitKey(delay);
 }
 
-int cv::createTrackbar(const string& trackbarName, const string& winName,
+int cv::waitKey(int delay)
+{
+    int code = waitKeyEx(delay);
+#ifndef HAVE_WINRT
+    static int use_legacy = -1;
+    if (use_legacy < 0)
+    {
+        use_legacy = getenv("OPENCV_LEGACY_WAITKEY") != NULL ? 1 : 0;
+    }
+    if (use_legacy > 0)
+        return code;
+#endif
+    return code & 0xff;
+}
+
+int cv::createTrackbar(const String& trackbarName, const String& winName,
                    int* value, int count, TrackbarCallback callback,
                    void* userdata)
 {
@@ -201,21 +229,36 @@ int cv::createTrackbar(const string& trackbarName, const string& winName,
                              value, count, callback, userdata);
 }
 
-void cv::setTrackbarPos( const string& trackbarName, const string& winName, int value )
+void cv::setTrackbarPos( const String& trackbarName, const String& winName, int value )
 {
     cvSetTrackbarPos(trackbarName.c_str(), winName.c_str(), value );
 }
 
-int cv::getTrackbarPos( const string& trackbarName, const string& winName )
+void cv::setTrackbarMax(const String& trackbarName, const String& winName, int maxval)
+{
+    cvSetTrackbarMax(trackbarName.c_str(), winName.c_str(), maxval);
+}
+
+void cv::setTrackbarMin(const String& trackbarName, const String& winName, int minval)
+{
+    cvSetTrackbarMin(trackbarName.c_str(), winName.c_str(), minval);
+}
+
+int cv::getTrackbarPos( const String& trackbarName, const String& winName )
 {
     return cvGetTrackbarPos(trackbarName.c_str(), winName.c_str());
 }
 
-void cv::setMouseCallback( const string& windowName, MouseCallback onMouse, void* param)
+void cv::setMouseCallback( const String& windowName, MouseCallback onMouse, void* param)
 {
     cvSetMouseCallback(windowName.c_str(), onMouse, param);
 }
 
+int cv::getMouseWheelDelta( int flags )
+{
+    return CV_GET_WHEEL_DELTA(flags);
+}
+
 int cv::startWindowThread()
 {
     return cvStartWindowThread();
@@ -223,17 +266,17 @@ int cv::startWindowThread()
 
 // OpenGL support
 
-void cv::setOpenGlDrawCallback(const string& name, OpenGlDrawCallback callback, void* userdata)
+void cv::setOpenGlDrawCallback(const String& name, OpenGlDrawCallback callback, void* userdata)
 {
     cvSetOpenGlDrawCallback(name.c_str(), callback, userdata);
 }
 
-void cv::setOpenGlContext(const string& windowName)
+void cv::setOpenGlContext(const String& windowName)
 {
     cvSetOpenGlContext(windowName.c_str());
 }
 
-void cv::updateWindow(const string& windowName)
+void cv::updateWindow(const String& windowName)
 {
     cvUpdateWindow(windowName.c_str());
 }
@@ -241,9 +284,9 @@ void cv::updateWindow(const string& windowName)
 #ifdef HAVE_OPENGL
 namespace
 {
-    std::map<std::string, cv::ogl::Texture2D> wndTexs;
-    std::map<std::string, cv::ogl::Texture2D> ownWndTexs;
-    std::map<std::string, cv::ogl::Buffer> ownWndBufs;
+    std::map<cv::String, cv::ogl::Texture2D> wndTexs;
+    std::map<cv::String, cv::ogl::Texture2D> ownWndTexs;
+    std::map<cv::String, cv::ogl::Buffer> ownWndBufs;
 
     void glDrawTextureCallback(void* userdata)
     {
@@ -254,14 +297,19 @@ namespace
 }
 #endif // HAVE_OPENGL
 
-void cv::imshow( const string& winname, InputArray _img )
+void cv::imshow( const String& winname, InputArray _img )
 {
+    const Size size = _img.size();
 #ifndef HAVE_OPENGL
-    Mat img = _img.getMat();
-    CvMat c_img = img;
-    cvShowImage(winname.c_str(), &c_img);
+    CV_Assert(size.width>0 && size.height>0);
+    {
+        Mat img = _img.getMat();
+        CvMat c_img = img;
+        cvShowImage(winname.c_str(), &c_img);
+    }
 #else
     const double useGl = getWindowProperty(winname, WND_PROP_OPENGL);
+    CV_Assert(size.width>0 && size.height>0);
 
     if (useGl <= 0)
     {
@@ -275,58 +323,72 @@ void cv::imshow( const string& winname, InputArray _img )
 
         if (autoSize > 0)
         {
-            Size size = _img.size();
             resizeWindow(winname, size.width, size.height);
         }
 
         setOpenGlContext(winname);
 
-        if (_img.kind() == _InputArray::OPENGL_TEXTURE)
+        cv::ogl::Texture2D& tex = ownWndTexs[winname];
+
+        if (_img.kind() == _InputArray::CUDA_GPU_MAT)
         {
-            cv::ogl::Texture2D& tex = wndTexs[winname];
-
-            tex = _img.getOGlTexture2D();
+            cv::ogl::Buffer& buf = ownWndBufs[winname];
+            buf.copyFrom(_img);
+            buf.setAutoRelease(false);
 
+            tex.copyFrom(buf);
             tex.setAutoRelease(false);
-
-            setOpenGlDrawCallback(winname, glDrawTextureCallback, &tex);
         }
         else
         {
-            cv::ogl::Texture2D& tex = ownWndTexs[winname];
-
-            if (_img.kind() == _InputArray::GPU_MAT)
-            {
-                cv::ogl::Buffer& buf = ownWndBufs[winname];
-                buf.copyFrom(_img);
-                buf.setAutoRelease(false);
-
-                tex.copyFrom(buf);
-                tex.setAutoRelease(false);
-            }
-            else
-            {
-                tex.copyFrom(_img);
-            }
-
-            tex.setAutoRelease(false);
-
-            setOpenGlDrawCallback(winname, glDrawTextureCallback, &tex);
+            tex.copyFrom(_img);
         }
 
+        tex.setAutoRelease(false);
+
+        setOpenGlDrawCallback(winname, glDrawTextureCallback, &tex);
+
         updateWindow(winname);
     }
 #endif
 }
 
-void cv::pointCloudShow(const string&, const GlCamera&, const GlArrays&)
+void cv::imshow(const String& winname, const ogl::Texture2D& _tex)
 {
-    CV_Error(CV_StsNotImplemented, "This function in deprecated, do not use it");
-}
+#ifndef HAVE_OPENGL
+    (void) winname;
+    (void) _tex;
+    CV_Error(cv::Error::OpenGlNotSupported, "The library is compiled without OpenGL support");
+#else
+    const double useGl = getWindowProperty(winname, WND_PROP_OPENGL);
 
-void cv::pointCloudShow(const string&, const GlCamera&, InputArray, InputArray)
-{
-    CV_Error(CV_StsNotImplemented, "This function in deprecated, do not use it");
+    if (useGl <= 0)
+    {
+        CV_Error(cv::Error::OpenGlNotSupported, "The window was created without OpenGL context");
+    }
+    else
+    {
+        const double autoSize = getWindowProperty(winname, WND_PROP_AUTOSIZE);
+
+        if (autoSize > 0)
+        {
+            Size size = _tex.size();
+            resizeWindow(winname, size.width, size.height);
+        }
+
+        setOpenGlContext(winname);
+
+        cv::ogl::Texture2D& tex = wndTexs[winname];
+
+        tex = _tex;
+
+        tex.setAutoRelease(false);
+
+        setOpenGlDrawCallback(winname, glDrawTextureCallback, &tex);
+
+        updateWindow(winname);
+    }
+#endif
 }
 
 // Without OpenGL
@@ -352,23 +414,33 @@ CV_IMPL void cvUpdateWindow(const char*)
 
 #if defined (HAVE_QT)
 
-CvFont cv::fontQt(const string& nameFont, int pointSize, Scalar color, int weight,  int style, int /*spacing*/)
+cv::QtFont cv::fontQt(const String& nameFont, int pointSize, Scalar color, int weight, int style, int spacing)
 {
-return cvFontQt(nameFont.c_str(), pointSize,color,weight, style);
+    CvFont f = cvFontQt(nameFont.c_str(), pointSize,color,weight, style, spacing);
+    void* pf = &f; // to suppress strict-aliasing
+    return *(cv::QtFont*)pf;
 }
 
-void cv::addText( const Mat& img, const string& text, Point org, CvFont font)
+void cv::addText( const Mat& img, const String& text, Point org, const QtFont& font)
 {
     CvMat _img = img;
-    cvAddText( &_img, text.c_str(), org,&font);
+    cvAddText( &_img, text.c_str(), org, (CvFont*)&font);
 }
 
-void cv::displayStatusBar(const string& name,  const string& text, int delayms)
+void cv::addText( const Mat& img, const String& text, Point org, const String& nameFont,
+        int pointSize, Scalar color, int weight, int style, int spacing)
+{
+    CvFont f = cvFontQt(nameFont.c_str(), pointSize,color,weight, style, spacing);
+    CvMat _img = img;
+    cvAddText( &_img, text.c_str(), org, &f);
+}
+
+void cv::displayStatusBar(const String& name,  const String& text, int delayms)
 {
     cvDisplayStatusBar(name.c_str(),text.c_str(), delayms);
 }
 
-void cv::displayOverlay(const string& name,  const string& text, int delayms)
+void cv::displayOverlay(const String& name,  const String& text, int delayms)
 {
     cvDisplayOverlay(name.c_str(),text.c_str(), delayms);
 }
@@ -383,78 +455,86 @@ void cv::stopLoop()
     cvStopLoop();
 }
 
-void cv::saveWindowParameters(const string& windowName)
+void cv::saveWindowParameters(const String& windowName)
 {
     cvSaveWindowParameters(windowName.c_str());
 }
 
-void cv::loadWindowParameters(const string& windowName)
+void cv::loadWindowParameters(const String& windowName)
 {
     cvLoadWindowParameters(windowName.c_str());
 }
 
-int cv::createButton(const string& button_name, ButtonCallback on_change, void* userdata, int button_type , bool initial_button_state  )
+int cv::createButton(const String& button_name, ButtonCallback on_change, void* userdata, int button_type , bool initial_button_state  )
 {
     return cvCreateButton(button_name.c_str(), on_change, userdata, button_type , initial_button_state );
 }
 
 #else
 
-CvFont cv::fontQt(const string&, int, Scalar, int,  int, int)
+static const char* NO_QT_ERR_MSG = "The library is compiled without QT support";
+
+cv::QtFont cv::fontQt(const String&, int, Scalar, int,  int, int)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
-    return CvFont();
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
+    return QtFont();
 }
 
-void cv::addText( const Mat&, const string&, Point, CvFont)
+void cv::addText( const Mat&, const String&, Point, const QtFont&)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
 }
 
-void cv::displayStatusBar(const string&,  const string&, int)
+void cv::addText(const Mat&, const String&, Point, const String&, int, Scalar, int, int, int)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
 }
 
-void cv::displayOverlay(const string&,  const string&, int )
+void cv::displayStatusBar(const String&,  const String&, int)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
+}
+
+void cv::displayOverlay(const String&,  const String&, int )
+{
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
 }
 
 int cv::startLoop(int (*)(int argc, char *argv[]), int , char**)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
     return 0;
 }
 
 void cv::stopLoop()
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
 }
 
-void cv::saveWindowParameters(const string&)
+void cv::saveWindowParameters(const String&)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
 }
 
-void cv::loadWindowParameters(const string&)
+void cv::loadWindowParameters(const String&)
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
 }
 
-int cv::createButton(const string&, ButtonCallback, void*, int , bool )
+int cv::createButton(const String&, ButtonCallback, void*, int , bool )
 {
-    CV_Error(CV_StsNotImplemented, "The library is compiled without QT support");
+    CV_Error(CV_StsNotImplemented, NO_QT_ERR_MSG);
     return 0;
 }
 
 #endif
 
-#if   defined(HAVE_WIN32UI)   // see window_w32.cpp
+#if   defined (HAVE_WIN32UI)  // see window_w32.cpp
 #elif defined (HAVE_GTK)      // see window_gtk.cpp
 #elif defined (HAVE_COCOA)    // see window_carbon.cpp
 #elif defined (HAVE_CARBON)
-#elif defined (HAVE_QT)       //YV see window_QT.cpp
+#elif defined (HAVE_QT)       // see window_QT.cpp
+#elif defined (WINRT) && !defined (WINRT_8_0) // see window_winrt.cpp
 
 #else
 
@@ -465,6 +545,12 @@ int cv::createButton(const string&, ButtonCallback, void*, int , bool )
 // version with a more capable one without a need to recompile dependent
 // applications or libraries.
 
+void cv::setWindowTitle(const String&, const String&)
+{
+    CV_Error(Error::StsNotImplemented, "The function is not implemented. "
+        "Rebuild the library with Windows, GTK+ 2.x or Carbon support. "
+        "If you are on Ubuntu or Debian, install libgtk2.0-dev and pkg-config, then re-run cmake or configure script");
+}
 
 #define CV_NO_GUI_ERROR(funcname) \
     cvError( CV_StsError, funcname, \
@@ -541,6 +627,16 @@ CV_IMPL void cvSetTrackbarPos( const char*, const char*, int )
     CV_NO_GUI_ERROR( "cvSetTrackbarPos" );
 }
 
+CV_IMPL void cvSetTrackbarMax(const char*, const char*, int)
+{
+    CV_NO_GUI_ERROR( "cvSetTrackbarMax" );
+}
+
+CV_IMPL void cvSetTrackbarMin(const char*, const char*, int)
+{
+    CV_NO_GUI_ERROR( "cvSetTrackbarMin" );
+}
+
 CV_IMPL void* cvGetWindowHandle( const char* )
 {
     CV_NO_GUI_ERROR( "cvGetWindowHandle" );
