@@ -40,21 +40,37 @@
 //M*/
 
 #include "precomp.hpp"
+#include "opencv2/imgproc.hpp"
 
 #ifndef WIN32
 
-#ifdef HAVE_GTK
+#if defined (HAVE_GTK)
 
-#include "gtk/gtk.h"
-#include "gdk/gdkkeysyms.h"
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <stdio.h>
 
+#if (GTK_MAJOR_VERSION == 3)
+  #define GTK_VERSION3 1
+#endif //GTK_MAJOR_VERSION >= 3
+#if (GTK_MAJOR_VERSION > 3 || (GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 4))
+  #define GTK_VERSION3_4 1
+#endif
+
 #ifdef HAVE_OPENGL
     #include <gtk/gtkgl.h>
     #include <GL/gl.h>
     #include <GL/glu.h>
 #endif
 
+#ifndef BIT_ALLIN
+    #define BIT_ALLIN(x,y) ( ((x)&(y)) == (y) )
+#endif
+#ifndef BIT_MAP
+    #define BIT_MAP(x,y,z) ( ((x)&(y)) ? (z) : 0 )
+#endif
+
 // TODO Fix the initial window size when flags=0.  Right now the initial window is by default
 // 320x240 size.  A better default would be actual size of the image.  Problem
 // is determining desired window size with trackbars while still allowing resizing.
@@ -88,14 +104,14 @@ GtkWidget*     cvImageWidgetNew      (int flags);
 void           cvImageWidgetSetImage(CvImageWidget * widget, const CvArr *arr);
 
 // standard GTK object macros
-#define CV_IMAGE_WIDGET(obj)          GTK_CHECK_CAST (obj, cvImageWidget_get_type (), CvImageWidget)
+#define CV_IMAGE_WIDGET(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, cvImageWidget_get_type (), CvImageWidget)
 #define CV_IMAGE_WIDGET_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, cvImageWidget_get_type (), CvImageWidgetClass)
-#define CV_IS_IMAGE_WIDGET(obj)       GTK_CHECK_TYPE (obj, cvImageWidget_get_type ())
+#define CV_IS_IMAGE_WIDGET(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, cvImageWidget_get_type ())
 
 /////////////////////////////////////////////////////////////////////////////
 // Private API ////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////
-GtkType        cvImageWidget_get_type (void);
+GType        cvImageWidget_get_type (void);
 
 static GtkWidgetClass * parent_class = NULL;
 
@@ -135,7 +151,7 @@ cvImageWidgetNew (int flags)
 {
   CvImageWidget *image_widget;
 
-  image_widget = CV_IMAGE_WIDGET( gtk_type_new (cvImageWidget_get_type ()) );
+  image_widget = CV_IMAGE_WIDGET( gtk_widget_new (cvImageWidget_get_type (), NULL) );
   image_widget->original_image = 0;
   image_widget->scaled_image = 0;
   image_widget->flags = flags | CV_WINDOW_NO_IMAGE;
@@ -149,32 +165,77 @@ cvImageWidget_realize (GtkWidget *widget)
   GdkWindowAttr attributes;
   gint attributes_mask;
 
+#if defined(GTK_VERSION3)
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+#endif //GTK_VERSION3
+
   //printf("cvImageWidget_realize\n");
   g_return_if_fail (widget != NULL);
   g_return_if_fail (CV_IS_IMAGE_WIDGET (widget));
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized(widget, TRUE);
 
+#if defined(GTK_VERSION3)
+  attributes.x = allocation.x;
+  attributes.y = allocation.y;
+  attributes.width = allocation.width;
+  attributes.height = allocation.height;
+#else
   attributes.x = widget->allocation.x;
   attributes.y = widget->allocation.y;
   attributes.width = widget->allocation.width;
   attributes.height = widget->allocation.height;
+#endif //GTK_VERSION3
+
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.event_mask = gtk_widget_get_events (widget) |
     GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK |
     GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK;
   attributes.visual = gtk_widget_get_visual (widget);
-  attributes.colormap = gtk_widget_get_colormap (widget);
 
+#if defined(GTK_VERSION3)
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
+  gtk_widget_set_window(
+      widget,
+      gdk_window_new(
+        gtk_widget_get_parent_window(widget),
+        &attributes,
+        attributes_mask
+        )
+      );
+
+  gtk_widget_set_style(
+      widget,
+      gtk_style_attach(
+        gtk_widget_get_style(widget),
+        gtk_widget_get_window(widget)
+        )
+      );
+
+  gdk_window_set_user_data (
+      gtk_widget_get_window(widget),
+      widget
+      );
+
+  gtk_style_set_background (
+      gtk_widget_get_style(widget),
+      gtk_widget_get_window(widget),
+      GTK_STATE_ACTIVE
+      );
+ #else
+  // The following lines are included to prevent breaking
+  // compatibility with older Gtk2 (<gtk+-2.18) libraries.
+  attributes.colormap = gtk_widget_get_colormap (widget);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
   widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);
 
   widget->style = gtk_style_attach (widget->style, widget->window);
-
   gdk_window_set_user_data (widget->window, widget);
 
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
+#endif // GTK_VERSION3
 }
 
 static CvSize cvImageWidget_calc_size( int im_width, int im_height, int max_width, int max_height ){
@@ -186,6 +247,56 @@ static CvSize cvImageWidget_calc_size( int im_width, int im_height, int max_widt
     return cvSize( cvRound(max_height*aspect), max_height );
 }
 
+#if defined (GTK_VERSION3)
+static void
+cvImageWidget_get_preferred_width (GtkWidget *widget, gint *minimal_width, gint *natural_width)
+{
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (CV_IS_IMAGE_WIDGET (widget));
+  CvImageWidget * image_widget = CV_IMAGE_WIDGET( widget );
+
+  if(image_widget->original_image != NULL) {
+    *minimal_width = (image_widget->flags & CV_WINDOW_AUTOSIZE) != CV_WINDOW_AUTOSIZE ?
+      gdk_window_get_width(gtk_widget_get_window(widget)) : image_widget->original_image->cols;
+  }
+  else {
+    *minimal_width = 320;
+  }
+
+  if(image_widget->scaled_image != NULL) {
+    *natural_width = *minimal_width < image_widget->scaled_image->cols ?
+      image_widget->scaled_image->cols : *minimal_width;
+  }
+  else {
+    *natural_width = *minimal_width;
+  }
+}
+
+static void
+cvImageWidget_get_preferred_height (GtkWidget *widget, gint *minimal_height, gint *natural_height)
+{
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (CV_IS_IMAGE_WIDGET (widget));
+  CvImageWidget * image_widget = CV_IMAGE_WIDGET( widget );
+
+  if(image_widget->original_image != NULL) {
+    *minimal_height = (image_widget->flags & CV_WINDOW_AUTOSIZE) != CV_WINDOW_AUTOSIZE ?
+      gdk_window_get_height(gtk_widget_get_window(widget)) : image_widget->original_image->rows;
+  }
+  else {
+    *minimal_height = 240;
+  }
+
+  if(image_widget->scaled_image != NULL) {
+    *natural_height = *minimal_height < image_widget->scaled_image->rows ?
+      image_widget->scaled_image->cols : *minimal_height;
+  }
+  else {
+    *natural_height = *minimal_height;
+  }
+}
+
+#else
 static void
 cvImageWidget_size_request (GtkWidget      *widget,
                        GtkRequisition *requisition)
@@ -216,6 +327,7 @@ cvImageWidget_size_request (GtkWidget      *widget,
     }
     //printf("%d %d\n",requisition->width, requisition->height);
 }
+#endif //GTK_VERSION3
 
 static void cvImageWidget_set_size(GtkWidget * widget, int max_width, int max_height){
     CvImageWidget * image_widget = CV_IMAGE_WIDGET( widget );
@@ -236,7 +348,7 @@ static void cvImageWidget_set_size(GtkWidget * widget, int max_width, int max_he
         cvReleaseMat( &image_widget->scaled_image );
     }
     if( !image_widget->scaled_image ){
-        image_widget->scaled_image = cvCreateMat( scaled_image_size.height, scaled_image_size.width,        CV_8UC3 );
+        image_widget->scaled_image = cvCreateMat( scaled_image_size.height, scaled_image_size.width, CV_8UC3 );
 
 
     }
@@ -254,7 +366,11 @@ cvImageWidget_size_allocate (GtkWidget     *widget,
   g_return_if_fail (CV_IS_IMAGE_WIDGET (widget));
   g_return_if_fail (allocation != NULL);
 
+#if defined (GTK_VERSION3)
+  gtk_widget_set_allocation(widget, allocation);
+#else
   widget->allocation = *allocation;
+#endif //GTK_VERSION3
   image_widget = CV_IMAGE_WIDGET (widget);
 
 
@@ -270,7 +386,7 @@ cvImageWidget_size_allocate (GtkWidget     *widget,
       cvResize( image_widget->original_image, image_widget->scaled_image, CV_INTER_AREA );
   }
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     {
       image_widget = CV_IMAGE_WIDGET (widget);
 
@@ -278,26 +394,37 @@ cvImageWidget_size_allocate (GtkWidget     *widget,
               ((image_widget->flags & CV_WINDOW_AUTOSIZE) ||
                (image_widget->flags & CV_WINDOW_NO_IMAGE)) )
       {
+#if defined (GTK_VERSION3)
+          allocation->width = image_widget->original_image->cols;
+          allocation->height = image_widget->original_image->rows;
+          gtk_widget_set_allocation(widget, allocation);
+#else
           widget->allocation.width = image_widget->original_image->cols;
           widget->allocation.height = image_widget->original_image->rows;
-          gdk_window_move_resize( widget->window, allocation->x, allocation->y,
-                  image_widget->original_image->cols, image_widget->original_image->rows );
+#endif //GTK_VERSION3
+          gdk_window_move_resize( gtk_widget_get_window(widget),
+              allocation->x, allocation->y,
+              image_widget->original_image->cols, image_widget->original_image->rows );
           if(image_widget->flags & CV_WINDOW_NO_IMAGE){
               image_widget->flags &= ~CV_WINDOW_NO_IMAGE;
               gtk_widget_queue_resize( GTK_WIDGET(widget) );
           }
       }
       else{
-          gdk_window_move_resize (widget->window,
+          gdk_window_move_resize (gtk_widget_get_window(widget),
                   allocation->x, allocation->y,
                   allocation->width, allocation->height );
-
       }
     }
 }
 
+#if defined (GTK_VERSION3)
+static void
+cvImageWidget_destroy (GtkWidget *object)
+#else
 static void
 cvImageWidget_destroy (GtkObject *object)
+#endif //GTK_VERSION3
 {
   CvImageWidget *image_widget;
 
@@ -309,24 +436,39 @@ cvImageWidget_destroy (GtkObject *object)
   cvReleaseMat( &image_widget->scaled_image );
   cvReleaseMat( &image_widget->original_image );
 
+#if defined (GTK_VERSION3)
+  if (GTK_WIDGET_CLASS (parent_class)->destroy)
+    (* GTK_WIDGET_CLASS (parent_class)->destroy) (object);
+#else
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+#endif //GTK_VERSION3
 }
 
 static void cvImageWidget_class_init (CvImageWidgetClass * klass)
 {
+#if defined (GTK_VERSION3)
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+#else
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
   object_class = (GtkObjectClass*) klass;
   widget_class = (GtkWidgetClass*) klass;
+#endif //GTK_VERSION3
 
-  parent_class = GTK_WIDGET_CLASS( gtk_type_class (gtk_widget_get_type ()) );
+  parent_class = GTK_WIDGET_CLASS( g_type_class_peek (gtk_widget_get_type ()) );
 
+#if defined (GTK_VERSION3)
+  widget_class->destroy = cvImageWidget_destroy;
+  widget_class->get_preferred_width = cvImageWidget_get_preferred_width;
+  widget_class->get_preferred_height = cvImageWidget_get_preferred_height;
+#else
   object_class->destroy = cvImageWidget_destroy;
+  widget_class->size_request = cvImageWidget_size_request;
+#endif //GTK_VERSION3
 
   widget_class->realize = cvImageWidget_realize;
-  widget_class->size_request = cvImageWidget_size_request;
   widget_class->size_allocate = cvImageWidget_size_allocate;
   widget_class->button_press_event = NULL;
   widget_class->button_release_event = NULL;
@@ -341,24 +483,20 @@ cvImageWidget_init (CvImageWidget *image_widget)
     image_widget->flags=0;
 }
 
-GtkType cvImageWidget_get_type (void){
-  static GtkType image_type = 0;
+GType cvImageWidget_get_type (void){
+  static GType image_type = 0;
 
   if (!image_type)
     {
-      static const GtkTypeInfo image_info =
-      {
-        (gchar*)"CvImageWidget",
-        sizeof (CvImageWidget),
-        sizeof (CvImageWidgetClass),
-        (GtkClassInitFunc) cvImageWidget_class_init,
-        (GtkObjectInitFunc) cvImageWidget_init,
-        /* reserved_1 */ NULL,
-        /* reserved_1 */ NULL,
-        (GtkClassInitFunc) NULL
-      };
-
-      image_type = gtk_type_unique (GTK_TYPE_WIDGET, &image_info);
+      image_type = g_type_register_static_simple(
+          GTK_TYPE_WIDGET,
+          (gchar*) "CvImageWidget",
+          sizeof(CvImageWidgetClass),
+          (GClassInitFunc) cvImageWidget_class_init,
+          sizeof(CvImageWidget),
+          (GInstanceInitFunc) cvImageWidget_init,
+          (GTypeFlags)0
+          );
     }
 
   return image_type;
@@ -380,6 +518,7 @@ typedef struct CvTrackbar
     int* data;
     int pos;
     int maxval;
+    int minval;
     CvTrackbarCallback notify;
     CvTrackbarCallback2 notify2;
     void* userdata;
@@ -449,8 +588,8 @@ CV_IMPL int cvInitSystem( int argc, char** argv )
     {
         hg_windows = 0;
 
-        gtk_disable_setlocale();
         gtk_init( &argc, &argv );
+        setlocale(LC_NUMERIC,"C");
 
         #ifdef HAVE_OPENGL
             gtk_gl_init(&argc, &argv);
@@ -480,9 +619,13 @@ CV_IMPL int cvStartWindowThread(){
     // conditional that indicates a key has been pressed
     cond_have_key = g_cond_new();
 
+#if !GLIB_CHECK_VERSION(2, 32, 0)
     // this is the window update thread
     window_thread = g_thread_create((GThreadFunc) icvWindowThreadLoop,
                     NULL, TRUE, NULL);
+#else
+    window_thread = g_thread_new("OpenCV window update", (GThreadFunc)icvWindowThreadLoop, NULL);
+#endif
     }
     thread_started = window_thread!=NULL;
     return thread_started;
@@ -605,6 +748,23 @@ void cvSetModeWindow_GTK( const char* name, double prop_value)//Yannick Verdie
     __END__;
 }
 
+void cv::setWindowTitle(const String& winname, const String& title)
+{
+    CvWindow* window = icvFindWindowByName(winname.c_str());
+
+    if (!window)
+    {
+        namedWindow(winname);
+        window = icvFindWindowByName(winname.c_str());
+    }
+
+    if (!window)
+        CV_Error(Error::StsNullPtr, "NULL window");
+
+    CV_LOCK_MUTEX();
+    gtk_window_set_title(GTK_WINDOW(window->frame), title.c_str());
+    CV_UNLOCK_MUTEX();
+}
 
 double cvGetPropWindowAutoSize_GTK(const char* name)
 {
@@ -647,8 +807,12 @@ double cvGetRatioWindow_GTK(const char* name)
     if (!window)
         EXIT; // keep silence here
 
+#if defined (GTK_VERSION3)
+    result = static_cast<double>(
+        gtk_widget_get_allocated_width(window->widget)) / gtk_widget_get_allocated_height(window->widget);
+#else
     result = static_cast<double>(window->widget->allocation.width) / window->widget->allocation.height;
-
+#endif // GTK_VERSION3
     __END__;
 
     return result;
@@ -743,7 +907,55 @@ namespace
 
 #endif // HAVE_OPENGL
 
+#if defined (GTK_VERSION3)
+static gboolean cvImageWidget_draw(GtkWidget* widget, cairo_t *cr, gpointer data)
+{
+#ifdef HAVE_OPENGL
+    CvWindow* window = (CvWindow*)data;
 
+    if (window->useGl)
+    {
+        drawGl(window);
+        return TRUE;
+    }
+#else
+    (void)data;
+#endif
+
+  CvImageWidget *image_widget = NULL;
+  GdkPixbuf *pixbuf = NULL;
+
+  g_return_val_if_fail (widget != NULL, FALSE);
+  g_return_val_if_fail (CV_IS_IMAGE_WIDGET (widget), FALSE);
+
+  image_widget = CV_IMAGE_WIDGET (widget);
+
+  if( image_widget->scaled_image ){
+      // center image in available region
+      int x0 = (gtk_widget_get_allocated_width(widget) - image_widget->scaled_image->cols)/2;
+      int y0 = (gtk_widget_get_allocated_height(widget) - image_widget->scaled_image->rows)/2;
+
+      pixbuf = gdk_pixbuf_new_from_data(image_widget->scaled_image->data.ptr, GDK_COLORSPACE_RGB, false,
+          8, MIN(image_widget->scaled_image->cols, gtk_widget_get_allocated_width(widget)),
+          MIN(image_widget->scaled_image->rows, gtk_widget_get_allocated_height(widget)),
+          image_widget->scaled_image->step, NULL, NULL);
+
+      gdk_cairo_set_source_pixbuf(cr, pixbuf, x0, y0);
+  }
+  else if( image_widget->original_image ){
+      pixbuf = gdk_pixbuf_new_from_data(image_widget->original_image->data.ptr, GDK_COLORSPACE_RGB, false,
+          8, MIN(image_widget->original_image->cols, gtk_widget_get_allocated_width(widget)),
+          MIN(image_widget->original_image->rows, gtk_widget_get_allocated_height(widget)),
+          image_widget->original_image->step, NULL, NULL);
+      gdk_cairo_set_source_pixbuf(cr, pixbuf, 0, 0);
+  }
+
+  cairo_paint(cr);
+  g_object_unref(pixbuf);
+  return TRUE;
+}
+
+#else
 static gboolean cvImageWidget_expose(GtkWidget* widget, GdkEventExpose* event, gpointer data)
 {
 #ifdef HAVE_OPENGL
@@ -758,7 +970,9 @@ static gboolean cvImageWidget_expose(GtkWidget* widget, GdkEventExpose* event, g
     (void)data;
 #endif
 
-  CvImageWidget *image_widget;
+  CvImageWidget *image_widget = NULL;
+  cairo_t *cr = NULL;
+  GdkPixbuf *pixbuf = NULL;
 
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (CV_IS_IMAGE_WIDGET (widget), FALSE);
@@ -767,31 +981,35 @@ static gboolean cvImageWidget_expose(GtkWidget* widget, GdkEventExpose* event, g
   if (event->count > 0)
     return FALSE;
 
+  cr = gdk_cairo_create(widget->window);
   image_widget = CV_IMAGE_WIDGET (widget);
 
-  gdk_window_clear_area (widget->window,
-                         0, 0,
-                         widget->allocation.width,
-                         widget->allocation.height);
   if( image_widget->scaled_image ){
       // center image in available region
       int x0 = (widget->allocation.width - image_widget->scaled_image->cols)/2;
       int y0 = (widget->allocation.height - image_widget->scaled_image->rows)/2;
 
-      gdk_draw_rgb_image( widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-          x0, y0, MIN(image_widget->scaled_image->cols, widget->allocation.width),
+      pixbuf = gdk_pixbuf_new_from_data(image_widget->scaled_image->data.ptr, GDK_COLORSPACE_RGB, false,
+          8, MIN(image_widget->scaled_image->cols, widget->allocation.width),
           MIN(image_widget->scaled_image->rows, widget->allocation.height),
-          GDK_RGB_DITHER_MAX, image_widget->scaled_image->data.ptr, image_widget->scaled_image->step );
+          image_widget->scaled_image->step, NULL, NULL);
+
+      gdk_cairo_set_source_pixbuf(cr, pixbuf, x0, y0);
   }
   else if( image_widget->original_image ){
-      gdk_draw_rgb_image( widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-          0, 0,
-          MIN(image_widget->original_image->cols, widget->allocation.width),
-           MIN(image_widget->original_image->rows, widget->allocation.height),
-          GDK_RGB_DITHER_MAX, image_widget->original_image->data.ptr, image_widget->original_image->step );
+      pixbuf = gdk_pixbuf_new_from_data(image_widget->original_image->data.ptr, GDK_COLORSPACE_RGB, false,
+          8, MIN(image_widget->original_image->cols, widget->allocation.width),
+          MIN(image_widget->original_image->rows, widget->allocation.height),
+          image_widget->original_image->step, NULL, NULL);
+      gdk_cairo_set_source_pixbuf(cr, pixbuf, 0, 0);
   }
+
+  cairo_paint(cr);
+  g_object_unref(pixbuf);
+  cairo_destroy(cr);
   return TRUE;
 }
+#endif //GTK_VERSION3
 
 CV_IMPL int cvNamedWindow( const char* name, int flags )
 {
@@ -802,6 +1020,7 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
 
     CvWindow* window;
     int len;
+    int b_nautosize;
 
     cvInitSystem(1,(char**)&name);
     if( !name )
@@ -854,20 +1073,32 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
     //
     // configure event handlers
     // TODO -- move this to CvImageWidget ?
-    gtk_signal_connect( GTK_OBJECT(window->frame), "key-press-event",
-                        GTK_SIGNAL_FUNC(icvOnKeyPress), window );
-    gtk_signal_connect( GTK_OBJECT(window->widget), "button-press-event",
-                        GTK_SIGNAL_FUNC(icvOnMouse), window );
-    gtk_signal_connect( GTK_OBJECT(window->widget), "button-release-event",
-                        GTK_SIGNAL_FUNC(icvOnMouse), window );
-    gtk_signal_connect( GTK_OBJECT(window->widget), "motion-notify-event",
-                        GTK_SIGNAL_FUNC(icvOnMouse), window );
-    gtk_signal_connect( GTK_OBJECT(window->frame), "delete-event",
-                        GTK_SIGNAL_FUNC(icvOnClose), window );
-    gtk_signal_connect( GTK_OBJECT(window->widget), "expose-event",
-                        GTK_SIGNAL_FUNC(cvImageWidget_expose), window );
+    g_signal_connect( window->frame, "key-press-event",
+                        G_CALLBACK(icvOnKeyPress), window );
+    g_signal_connect( window->widget, "button-press-event",
+                        G_CALLBACK(icvOnMouse), window );
+    g_signal_connect( window->widget, "button-release-event",
+                        G_CALLBACK(icvOnMouse), window );
+    g_signal_connect( window->widget, "motion-notify-event",
+                        G_CALLBACK(icvOnMouse), window );
+    g_signal_connect( window->widget, "scroll-event",
+                        G_CALLBACK(icvOnMouse), window );
+    g_signal_connect( window->frame, "delete-event",
+                        G_CALLBACK(icvOnClose), window );
+#if defined(GTK_VERSION3)
+    g_signal_connect( window->widget, "draw",
+                        G_CALLBACK(cvImageWidget_draw), window );
+#else
+    g_signal_connect( window->widget, "expose-event",
+                        G_CALLBACK(cvImageWidget_expose), window );
+#endif //GTK_VERSION3
 
-    gtk_widget_add_events (window->widget, GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK) ;
+
+#if defined(GTK_VERSION3_4)
+    gtk_widget_add_events (window->widget, GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK | GDK_SMOOTH_SCROLL_MASK) ;
+#else
+    gtk_widget_add_events (window->widget, GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK) ;
+#endif //GTK_VERSION3_4
 
     gtk_widget_show( window->frame );
     gtk_window_set_title( GTK_WINDOW(window->frame), name );
@@ -876,11 +1107,11 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
         hg_windows->prev = window;
     hg_windows = window;
 
-    gtk_window_set_resizable( GTK_WINDOW(window->frame), (flags & CV_WINDOW_AUTOSIZE) == 0 );
-
+    b_nautosize = ((flags & CV_WINDOW_AUTOSIZE) == 0);
+    gtk_window_set_resizable( GTK_WINDOW(window->frame), b_nautosize );
 
     // allow window to be resized
-    if( (flags & CV_WINDOW_AUTOSIZE)==0 ){
+    if( b_nautosize ){
         GdkGeometry geometry;
         geometry.min_width = 50;
         geometry.min_height = 50;
@@ -1007,13 +1238,33 @@ static void icvDeleteWindow( CvWindow* window )
     }
 
     cvFree( &window );
+
+    // if last window...
+    if( hg_windows == 0 )
+    {
 #ifdef HAVE_GTHREAD
-    // if last window, send key press signal
-    // to jump out of any waiting cvWaitKey's
-    if(hg_windows==0 && thread_started){
-        g_cond_broadcast(cond_have_key);
-    }
+        if( thread_started )
+        {
+            // send key press signal to jump out of any waiting cvWaitKey's
+            g_cond_broadcast( cond_have_key );
+        }
+        else
+        {
 #endif
+            // Some GTK+ modules (like the Unity module) use GDBusConnection,
+            // which has a habit of postponing cleanup by performing it via
+            // idle sources added to the main loop. Since this was the last window,
+            // we can assume that no event processing is going to happen in the
+            // nearest future, so we should force that cleanup (by handling all pending
+            // events) while we still have the chance.
+            // This is not needed if thread_started is true, because the background
+            // thread will process events continuously.
+            while( gtk_events_pending() )
+                gtk_main_iteration();
+#ifdef HAVE_GTHREAD
+        }
+#endif
+    }
 }
 
 
@@ -1096,10 +1347,7 @@ cvShowImage( const char* name, const CvArr* arr )
     #ifdef HAVE_OPENGL
         if (window->useGl)
         {
-            CvMat stub;
-            CvMat* mat = cvGetMat(arr, &stub);
-            cv::Mat im(mat);
-            cv::imshow(name, im);
+            cv::imshow(name, cv::cvarrToMat(arr));
             return;
         }
     #endif
@@ -1228,7 +1476,6 @@ icvCreateTrackbar( const char* trackbar_name, const char* window_name,
         GtkWidget* hscale_box = gtk_hbox_new( FALSE, 10 );
         GtkWidget* hscale_label = gtk_label_new( trackbar_name );
         GtkWidget* hscale = gtk_hscale_new_with_range( 0, count, 1 );
-        gtk_range_set_update_policy( GTK_RANGE(hscale), GTK_UPDATE_CONTINUOUS );
         gtk_scale_set_digits( GTK_SCALE(hscale), 0 );
         //gtk_scale_set_value_pos( hscale, GTK_POS_TOP );
         gtk_scale_set_draw_value( GTK_SCALE(hscale), TRUE );
@@ -1259,8 +1506,8 @@ icvCreateTrackbar( const char* trackbar_name, const char* window_name,
     trackbar->notify = on_notify;
     trackbar->notify2 = on_notify2;
     trackbar->userdata = userdata;
-    gtk_signal_connect( GTK_OBJECT(trackbar->widget), "value-changed",
-                        GTK_SIGNAL_FUNC(icvOnTrackbar), trackbar );
+    g_signal_connect( trackbar->widget, "value-changed",
+                        G_CALLBACK(icvOnTrackbar), trackbar );
 
     // queue a widget resize to trigger a window resize to
     // compensate for the addition of trackbars
@@ -1381,6 +1628,80 @@ CV_IMPL void cvSetTrackbarPos( const char* trackbar_name, const char* window_nam
 }
 
 
+CV_IMPL void cvSetTrackbarMax(const char* trackbar_name, const char* window_name, int maxval)
+{
+    CV_FUNCNAME("cvSetTrackbarMax");
+
+    __BEGIN__;
+
+    if (maxval >= 0)
+    {
+        CvWindow* window = 0;
+        CvTrackbar* trackbar = 0;
+
+        if (trackbar_name == 0 || window_name == 0)
+        {
+            CV_ERROR( CV_StsNullPtr, "NULL trackbar or window name");
+        }
+
+        window = icvFindWindowByName( window_name );
+        if (window)
+        {
+            trackbar = icvFindTrackbarByName(window, trackbar_name);
+            if (trackbar)
+            {
+                trackbar->maxval = (trackbar->minval>maxval)?trackbar->minval:maxval;
+
+                CV_LOCK_MUTEX();
+
+                gtk_range_set_range(GTK_RANGE(trackbar->widget), 0, trackbar->maxval);
+
+                CV_UNLOCK_MUTEX();
+            }
+        }
+    }
+
+    __END__;
+}
+
+
+CV_IMPL void cvSetTrackbarMin(const char* trackbar_name, const char* window_name, int minval)
+{
+    CV_FUNCNAME("cvSetTrackbarMin");
+
+    __BEGIN__;
+
+    if (minval >= 0)
+    {
+        CvWindow* window = 0;
+        CvTrackbar* trackbar = 0;
+
+        if (trackbar_name == 0 || window_name == 0)
+        {
+            CV_ERROR( CV_StsNullPtr, "NULL trackbar or window name");
+        }
+
+        window = icvFindWindowByName( window_name );
+        if (window)
+        {
+            trackbar = icvFindTrackbarByName(window, trackbar_name);
+            if (trackbar)
+            {
+                trackbar->minval = (minval<trackbar->maxval)?minval:trackbar->maxval;
+
+                CV_LOCK_MUTEX();
+
+                gtk_range_set_range(GTK_RANGE(trackbar->widget), minval, trackbar->maxval);
+
+                CV_UNLOCK_MUTEX();
+            }
+        }
+    }
+
+    __END__;
+}
+
+
 CV_IMPL void* cvGetWindowHandle( const char* window_name )
 {
     void* widget = 0;
@@ -1426,11 +1747,110 @@ CV_IMPL const char* cvGetWindowName( void* window_handle )
     return window_name;
 }
 
-static gboolean icvOnKeyPress( GtkWidget * /*widget*/,
-                GdkEventKey* event, gpointer /*user_data*/ )
+static GtkFileFilter* icvMakeGtkFilter(const char* name, const char* patterns, GtkFileFilter* images)
+{
+    GtkFileFilter* filter = gtk_file_filter_new();
+    gtk_file_filter_set_name(filter, name);
+
+    while(patterns[0])
+    {
+        gtk_file_filter_add_pattern(filter, patterns);
+        gtk_file_filter_add_pattern(images, patterns);
+        patterns += strlen(patterns) + 1;
+    }
+
+    return filter;
+}
+
+static void icvShowSaveAsDialog(GtkWidget* widget, CvWindow* window)
+{
+    if (!window || !widget)
+        return;
+
+    CvImageWidget* image_widget = CV_IMAGE_WIDGET(window->widget);
+    if (!image_widget || !image_widget->original_image)
+        return;
+
+    GtkWidget* dialog = gtk_file_chooser_dialog_new("Save As...",
+                      GTK_WINDOW(widget),
+                      GTK_FILE_CHOOSER_ACTION_SAVE,
+                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                      NULL);
+    gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
+
+    cv::String sname = gtk_window_get_title(GTK_WINDOW(window->frame));
+    sname = sname.substr(sname.find_last_of("\\/") + 1) + ".png";
+    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), sname.c_str());
+
+    GtkFileFilter* filter_all = gtk_file_filter_new();
+    gtk_file_filter_set_name(filter_all, "All Files");
+    gtk_file_filter_add_pattern(filter_all, "*");
+
+    GtkFileFilter* filter_images = gtk_file_filter_new();
+    gtk_file_filter_set_name(filter_images, "All Images");
+
+    GtkFileFilter* file_filters[] = {
+        icvMakeGtkFilter("Portable Network Graphics files (*.png)",               "*.png\0",                             filter_images),
+        icvMakeGtkFilter("JPEG files (*.jpeg;*.jpg;*.jpe)",                       "*.jpeg\0*.jpg\0*.jpe\0",              filter_images),
+        icvMakeGtkFilter("Windows bitmap (*.bmp;*.dib)",                          "*.bmp\0*.dib\0",                      filter_images),
+        icvMakeGtkFilter("TIFF Files (*.tiff;*.tif)",                             "*.tiff\0*.tif\0",                     filter_images),
+        icvMakeGtkFilter("JPEG-2000 files (*.jp2)",                               "*.jp2\0",                             filter_images),
+        icvMakeGtkFilter("WebP files (*.webp)",                                   "*.webp\0",                            filter_images),
+        icvMakeGtkFilter("Portable image format (*.pbm;*.pgm;*.ppm;*.pxm;*.pnm)", "*.pbm\0*.pgm\0*.ppm\0*.pxm\0*.pnm\0", filter_images),
+        icvMakeGtkFilter("OpenEXR Image files (*.exr)",                           "*.exr\0",                             filter_images),
+        icvMakeGtkFilter("Radiance HDR (*.hdr;*.pic)",                            "*.hdr\0*.pic\0",                      filter_images),
+        icvMakeGtkFilter("Sun raster files (*.sr;*.ras)",                         "*.sr\0*.ras\0",                       filter_images),
+        filter_images,
+        filter_all
+    };
+
+    for (size_t idx = 0; idx < sizeof(file_filters)/sizeof(file_filters[0]); ++idx)
+        gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), file_filters[idx]);
+    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter_images);
+
+    cv::String filename;
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
+    {
+        char* fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+        filename = fname;
+        g_free(fname);
+    }
+    gtk_widget_destroy(dialog);
+
+    if (!filename.empty())
+    {
+        cv::Mat bgr;
+        cv::cvtColor(cv::cvarrToMat(image_widget->original_image), bgr, cv::COLOR_RGB2BGR);
+        cv::imwrite(filename, bgr);
+    }
+}
+
+#if defined (GTK_VERSION3)
+#define GDK_Escape GDK_KEY_Escape
+#define GDK_Return GDK_KEY_Return
+#define GDK_Linefeed GDK_KEY_Linefeed
+#define GDK_Tab GDK_KEY_Tab
+#define GDK_s GDK_KEY_s
+#define GDK_S GDK_KEY_S
+#endif //GTK_VERSION3
+
+static gboolean icvOnKeyPress(GtkWidget* widget, GdkEventKey* event, gpointer user_data)
 {
     int code = 0;
 
+    if ( BIT_ALLIN(event->state, GDK_CONTROL_MASK) && (event->keyval == GDK_s || event->keyval == GDK_S))
+    {
+        try
+        {
+            icvShowSaveAsDialog(widget, (CvWindow*)user_data);
+        }
+        catch(...)
+        {
+            // suppress all exceptions here
+        }
+    }
+
     switch( event->keyval )
     {
     case GDK_Escape:
@@ -1501,9 +1921,9 @@ static gboolean icvOnMouse( GtkWidget *widget, GdkEvent *event, gpointer user_da
 {
     // TODO move this logic to CvImageWidget
     CvWindow* window = (CvWindow*)user_data;
-    CvPoint2D32f pt32f = {-1., -1.};
-    CvPoint pt = {-1,-1};
-    int cv_event = -1, state = 0;
+    CvPoint2D32f pt32f(-1., -1.);
+    CvPoint pt(-1,-1);
+    int cv_event = -1, state = 0, flags = 0;
     CvImageWidget * image_widget = CV_IMAGE_WIDGET( widget );
 
     if( window->signature != CV_WINDOW_MAGIC_VAL ||
@@ -1549,39 +1969,74 @@ static gboolean icvOnMouse( GtkWidget *widget, GdkEvent *event, gpointer user_da
         }
         state = event_button->state;
     }
+    else if( event->type == GDK_SCROLL )
+    {
+#if defined(GTK_VERSION3_4)
+        // NOTE: in current implementation doesn't possible to put into callback function delta_x and delta_y separetely
+        double delta = (event->scroll.delta_x + event->scroll.delta_y);
+        cv_event   = (event->scroll.delta_y!=0) ? CV_EVENT_MOUSEHWHEEL : CV_EVENT_MOUSEWHEEL;
+#else
+        cv_event = CV_EVENT_MOUSEWHEEL;
+#endif //GTK_VERSION3_4
 
-    if( cv_event >= 0 ){
+        state    = event->scroll.state;
+
+        switch(event->scroll.direction) {
+#if defined(GTK_VERSION3_4)
+        case GDK_SCROLL_SMOOTH: flags |= (((int)delta << 16));
+            break;
+#endif //GTK_VERSION3_4
+        case GDK_SCROLL_LEFT:  cv_event = CV_EVENT_MOUSEHWHEEL;
+        case GDK_SCROLL_UP:    flags |= ~0xffff;
+            break;
+        case GDK_SCROLL_RIGHT: cv_event = CV_EVENT_MOUSEHWHEEL;
+        case GDK_SCROLL_DOWN:  flags |= (((int)1 << 16));
+            break;
+        default: ;
+        };
+    }
+
+    if( cv_event >= 0 )
+    {
         // scale point if image is scaled
         if( (image_widget->flags & CV_WINDOW_AUTOSIZE)==0 &&
              image_widget->original_image &&
-             image_widget->scaled_image ){
+             image_widget->scaled_image )
+        {
             // image origin is not necessarily at (0,0)
+#if defined (GTK_VERSION3)
+            int x0 = (gtk_widget_get_allocated_width(widget) - image_widget->scaled_image->cols)/2;
+            int y0 = (gtk_widget_get_allocated_height(widget) - image_widget->scaled_image->rows)/2;
+#else
             int x0 = (widget->allocation.width - image_widget->scaled_image->cols)/2;
             int y0 = (widget->allocation.height - image_widget->scaled_image->rows)/2;
-            pt.x = cvRound( ((pt32f.x-x0)*image_widget->original_image->cols)/
+#endif //GTK_VERSION3
+            pt.x = cvFloor( ((pt32f.x-x0)*image_widget->original_image->cols)/
                                             image_widget->scaled_image->cols );
-            pt.y = cvRound( ((pt32f.y-y0)*image_widget->original_image->rows)/
+            pt.y = cvFloor( ((pt32f.y-y0)*image_widget->original_image->rows)/
                                             image_widget->scaled_image->rows );
         }
-        else{
+        else
+        {
             pt = cvPointFrom32f( pt32f );
         }
 
 //        if((unsigned)pt.x < (unsigned)(image_widget->original_image->width) &&
 //           (unsigned)pt.y < (unsigned)(image_widget->original_image->height) )
         {
-            int flags = (state & GDK_SHIFT_MASK ? CV_EVENT_FLAG_SHIFTKEY : 0) |
-                (state & GDK_CONTROL_MASK ? CV_EVENT_FLAG_CTRLKEY : 0) |
-                (state & (GDK_MOD1_MASK|GDK_MOD2_MASK) ? CV_EVENT_FLAG_ALTKEY : 0) |
-                (state & GDK_BUTTON1_MASK ? CV_EVENT_FLAG_LBUTTON : 0) |
-                (state & GDK_BUTTON2_MASK ? CV_EVENT_FLAG_MBUTTON : 0) |
-                (state & GDK_BUTTON3_MASK ? CV_EVENT_FLAG_RBUTTON : 0);
+            flags |= BIT_MAP(state, GDK_SHIFT_MASK,   CV_EVENT_FLAG_SHIFTKEY) |
+                BIT_MAP(state, GDK_CONTROL_MASK, CV_EVENT_FLAG_CTRLKEY)  |
+                BIT_MAP(state, GDK_MOD1_MASK,    CV_EVENT_FLAG_ALTKEY)   |
+                BIT_MAP(state, GDK_MOD2_MASK,    CV_EVENT_FLAG_ALTKEY)   |
+                BIT_MAP(state, GDK_BUTTON1_MASK, CV_EVENT_FLAG_LBUTTON)  |
+                BIT_MAP(state, GDK_BUTTON2_MASK, CV_EVENT_FLAG_MBUTTON)  |
+                BIT_MAP(state, GDK_BUTTON3_MASK, CV_EVENT_FLAG_RBUTTON);
             window->on_mouse( cv_event, pt.x, pt.y, flags, window->on_mouse_param );
         }
     }
 
-        return FALSE;
-    }
+    return FALSE;
+}
 
 
 static gboolean icvAlarm( gpointer user_data )
