@@ -20,14 +20,12 @@
 #  define LOGD(...)
 #endif
 
+#include "opencv2/core/utility.hpp"
+
 #include "converters.h"
 
-#include "core_manual.hpp"
-#include "features2d_manual.hpp"
-
-
 #ifdef _MSC_VER
 #  pragma warning(disable:4800 4244)
 #endif
 
-#endif //__JAVA_COMMON_H__
\ No newline at end of file
+#endif //__JAVA_COMMON_H__
