@@ -4,10 +4,10 @@
  * @author
  */
 
-#include "opencv2/highgui/highgui.hpp"
-#include "opencv2/imgproc/imgproc.hpp"
+#include "opencv2/highgui.hpp"
+#include "opencv2/imgcodecs.hpp"
+#include "opencv2/imgproc.hpp"
 #include <iostream>
-#include <stdio.h>
 
 using namespace std;
 using namespace cv;
@@ -15,14 +15,21 @@ using namespace cv;
 /**
  * @function main
  */
-int main( int, char** argv )
+int main(int argc, char** argv)
 {
   Mat src, dst;
 
   /// Load image
-  src = imread( argv[1], 1 );
+  String imageName( "../data/lena.jpg" ); // by default
 
-  if( !src.data )
+  if (argc > 1)
+  {
+      imageName = argv[1];
+  }
+
+  src = imread( imageName, IMREAD_COLOR );
+
+  if( src.empty() )
     { return -1; }
 
   /// Separate the image in 3 places ( B, G and R )
@@ -71,7 +78,7 @@ int main( int, char** argv )
   }
 
   /// Display
-  namedWindow("calcHist Demo", CV_WINDOW_AUTOSIZE );
+  namedWindow("calcHist Demo", WINDOW_AUTOSIZE );
   imshow("calcHist Demo", histImage );
 
   waitKey(0);
